/////////////////////////////////////////////////////////////////////////////
// GliD3D_Config_SpecialEffect.cpp : No comment
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Directive

#include "StdHdr.h"
#pragma hdrstop
#include "GliD3D.h"

/////////////////////////////////////////////////////////////////////////////
// Defines / Variables

/////////////////////////////////////////////////////////////////////////////
// Prototypes

/////////////////////////////////////////////////////////////////////////////
// Implements

int
grCheckTextureOn (GrCombineFunction_t function, GrCombineFactor_t factor,
		  GrCombineOther_t other)
{
  if (other == GR_COMBINE_OTHER_TEXTURE)
    {
      //return 1;
      switch (function)
	{
	case GR_COMBINE_FUNCTION_SCALE_OTHER:
	case GR_COMBINE_FUNCTION_SCALE_OTHER_ADD_LOCAL:
	case GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL:
	case GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL_ADD_LOCAL:
	case GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL_ADD_LOCAL_ALPHA:
	  return 1;
	default:
	  return 0;
	}
    }
  else
    {
      return 0;
    }
}

inline VOID
CombineLocalIterated (VERTEX2 & ptlv, const GrVertex * pv)
{
  switch (g_gr_state.alpha_combine_local)
    {
    case GR_COMBINE_LOCAL_ITERATED:
      ptlv.r = pv->r / 255.0f;
      ptlv.b = pv->b / 255.0f;
      ptlv.g = pv->g / 255.0f;
      ptlv.a = pv->a / 255.0f;
      break;
    case GR_COMBINE_LOCAL_CONSTANT:
      ptlv.r = pv->r / 255.0f;
      ptlv.b = pv->b / 255.0f;
      ptlv.g = pv->g / 255.0f;
      ptlv.a = (g_gr_state.constant_color_value_alpha >> 24) / 255.0f;
      break;
    }
}

inline VOID
CombineLocalConstant (VERTEX2 & ptlv, const GrVertex * pv)
{
  switch (g_gr_state.alpha_combine_local)
    {
    case GR_COMBINE_LOCAL_ITERATED:
      ptlv.r = ((g_gr_state.constant_color_value_color & 0xff0000) >> 16) / 255.0f;
      ptlv.g = ((g_gr_state.constant_color_value_color & 0x00ff00) >> 8) / 255.0f;
      ptlv.b = ((g_gr_state.constant_color_value_color & 0x0000ff) >> 0) / 255.0f;
      ptlv.a = pv->a / 255.0f;
      break;
    case GR_COMBINE_LOCAL_CONSTANT:
      ptlv.r = ((g_gr_state.constant_color_value_color & 0xff0000) >> 16) / 255.0f;
      ptlv.g = ((g_gr_state.constant_color_value_color & 0x00ff00) >> 8) / 255.0f;
      ptlv.b = ((g_gr_state.constant_color_value_color & 0x0000ff) >> 0) / 255.0f;
      ptlv.a = (g_gr_state.constant_color_value_alpha >> 24) / 255.0f;
      break;
    }
}

enum CombineFunc_t
{
  C_ITERATED,
  C_CONSTANT,
  C_NONE
};


template < bool OriginLeft, CombineFunc_t func >
VOID VertexConv (VERTEX2 & ptlv, const GrVertex * pv)
{
  if (pv->x < 4096)
    ptlv.cord.x = pv->x;
  else
    ptlv.cord.x = pv->x - (FLOAT) (((DWORD) pv->x) & 0xfffff000);
  if (OriginLeft)
  {
      if (pv->y < 4096)
	    ptlv.cord.y = pv->y;
      else
	    ptlv.cord.y = pv->y - (FLOAT) (((DWORD) pv->y) & 0xfffff000);
  } else {
      if (pv->y < 4096)
	    ptlv.cord.y = (FLOAT) g_gr_nHeight - pv->y;
      else
	    ptlv.cord.y = (FLOAT) g_gr_nHeight - (FLOAT) (((DWORD) pv->y) & 0xfffff000);
  }
  ptlv.cord.w = 1.0 / pv->oow;
  if (g_gr_bUseZInverse)
    {
      if (g_gr_bPrecisionFix)
      {
	  if (ptlv.cord.w < 10)
	    ptlv.cord.w = 10;

      }
      ptlv.cord.z = 1.0f;
    }
  else
    {
      ptlv.cord.z = -ptlv.cord.w * ptlv.cord.w;
    }

  ptlv.cord.x *= ptlv.cord.w;
  ptlv.cord.y *= ptlv.cord.w;

  switch (func)
    {
    case C_ITERATED:
      CombineLocalIterated (ptlv, pv);
      break;
    case C_CONSTANT:
      CombineLocalConstant (ptlv, pv);
      break;
    case C_NONE:
      ptlv.r = 0;
      ptlv.g = 0;
      ptlv.b = 0;
      ptlv.a = 0;
      break;
    }

  // set color to (1,1,1) if using texture scaled with factor 1
  // and (0,0,0) if using texture scaled with factor 0
  if (g_gr_state.color_combine_function == GR_COMBINE_FUNCTION_SCALE_OTHER ||
      g_gr_state.color_combine_function == GR_COMBINE_FUNCTION_SCALE_OTHER_ADD_LOCAL)
    {
      if (g_gr_state.color_combine_other == GR_COMBINE_OTHER_TEXTURE)
	{
	  if (g_gr_state.color_combine_factor == GR_COMBINE_FACTOR_ONE)
	    {
	      ptlv.r = 1;
	      ptlv.g = 1;
	      ptlv.b = 1;
	    }
	  else if (g_gr_state.color_combine_factor == GR_COMBINE_FACTOR_ZERO)
	    {
	      ptlv.r = 0;
	      ptlv.g = 0;
	      ptlv.b = 0;
	    }
	}
    }

  if (g_gr_state.alpha_combine_function == GR_COMBINE_FUNCTION_SCALE_OTHER &&
      g_gr_state.alpha_combine_other == GR_COMBINE_OTHER_TEXTURE)
    {
      if (g_gr_state.alpha_combine_factor == GR_COMBINE_FACTOR_ONE)
	ptlv.a = 1;
      else if (g_gr_state.alpha_combine_factor == GR_COMBINE_FACTOR_ZERO)
	ptlv.a = 0;
    }


  ptlv.tu = ((pv->tmuvtx[0].sow / pv->oow) + 0.2f) / 256.0;
  ptlv.tv = ((pv->tmuvtx[0].tow / pv->oow) + 0.2f) / 256.0;


  // scale texture coordinates according to aspect ratio
  INT nTmu;

//      nTmu = grsubGetTmuIndex(tmu);
  nTmu = 0;

  int ah, aw;
  grsubAspectToInt (aw, ah, g_gr_tmu[nTmu].texture_source_info.aspectRatio);
  ptlv.tu *= ah;
  ptlv.tv *= aw;

}
void
notcalled (void)
{
  // to solve a BC++ bug
  VERTEX2 a;
  GrVertex *v = NULL;
  VertexConv < false, C_ITERATED > (a, v);
}


static vertexconv_func lookup_vfunc[2][3];
vertexconv_func grsubSetVertexFromGrVertex;

void
InitLookupVertexFuncs (void)
{
#define DEF_FUNC(a,b) lookup_vfunc[a][b]=VertexConv<a,b>
  DEF_FUNC (true, C_ITERATED);
  DEF_FUNC (false, C_ITERATED);
  DEF_FUNC (true, C_CONSTANT);
  DEF_FUNC (false, C_CONSTANT);
  DEF_FUNC (true, C_NONE);
  DEF_FUNC (false, C_NONE);
#undef DEF_FUNC
}

void
ChooseVertexFunc (void)
{
  CombineFunc_t func;
  switch (g_gr_state.color_combine_local)
    {
    case GR_COMBINE_LOCAL_ITERATED:
      func = C_ITERATED;
      break;
    case GR_COMBINE_LOCAL_CONSTANT:
      if (			//g_gr_state.color_combine_function==GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL_ADD_LOCAL &&
	   g_gr_state.color_combine_factor == GR_COMBINE_FACTOR_TEXTURE_RGB &&
	   g_gr_state.color_combine_other == GR_COMBINE_OTHER_ITERATED)
	func = C_ITERATED;
      else
	func = C_CONSTANT;
      break;
    default:
      func = C_NONE;
      break;
    }
  bool OriginLeft = (g_gr_state.origin_location == GR_ORIGIN_UPPER_LEFT);
  grsubSetVertexFromGrVertex = lookup_vfunc[OriginLeft][func];
}

void
grMegaCombineEverythingSetup (bool flag)
{
  ChooseVertexFunc ();
  switch (g_gr_state.color_combine_function)
    {
    case GR_COMBINE_FUNCTION_LOCAL:
    case GR_COMBINE_FUNCTION_ZERO:
      glDisable (GL_TEXTURE_2D);
      break;

    case GR_COMBINE_FUNCTION_LOCAL_ALPHA:
      glDisable (GL_TEXTURE_2D);
      break;

    case GR_COMBINE_FUNCTION_SCALE_OTHER_ADD_LOCAL:
    case GR_COMBINE_FUNCTION_SCALE_OTHER:
      if (g_gr_state.color_combine_other == GR_COMBINE_OTHER_TEXTURE)
	glEnable (GL_TEXTURE_2D);
      else
	glDisable (GL_TEXTURE_2D);
      switch (g_gr_state.color_combine_factor)
	{
	case GR_COMBINE_FACTOR_NONE:
	  glDisable (GL_TEXTURE_2D);
	  break;
	case GR_COMBINE_FACTOR_ONE:
	case GR_COMBINE_FACTOR_LOCAL:
	  break;
	case GR_COMBINE_FACTOR_LOCAL_ALPHA:
	  break;
	default:
	  glDisable (GL_TEXTURE_2D);
	  break;
	}
      break;

    case GR_COMBINE_FUNCTION_SCALE_OTHER_ADD_LOCAL_ALPHA:
      if (g_gr_state.color_combine_other == GR_COMBINE_OTHER_TEXTURE)
	glEnable (GL_TEXTURE_2D);
      else
	glDisable (GL_TEXTURE_2D);

      break;


    case GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL:
    case GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL_ADD_LOCAL:
    case GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL_ADD_LOCAL_ALPHA:
    case GR_COMBINE_FUNCTION_SCALE_MINUS_LOCAL_ADD_LOCAL:
      if (g_gr_state.color_combine_other == GR_COMBINE_OTHER_TEXTURE)
	glEnable (GL_TEXTURE_2D);
      else
	glDisable (GL_TEXTURE_2D);
      break;
    case GR_COMBINE_FUNCTION_SCALE_MINUS_LOCAL_ADD_LOCAL_ALPHA:
      glDisable (GL_TEXTURE_2D);
      break;
    }

  if (flag)
    grAlphaBlendFunction (g_gr_state.alpha_blend_function_rgb_sf,
			  g_gr_state.alpha_blend_function_rgb_df,
			  g_gr_state.alpha_blend_function_alpha_sf,
			  g_gr_state.alpha_blend_function_alpha_df);

}


FX_ENTRY void FX_CALL
grAlphaBlendFunction (
		       GrAlphaBlendFnc_t rgb_sf, GrAlphaBlendFnc_t rgb_df,
		       GrAlphaBlendFnc_t alpha_sf, GrAlphaBlendFnc_t alpha_df
)
{

  GLenum dfactor, sfactor;

  TRACEFUNCTION ("grAlphaBlendFunction");

  g_gr_state.alpha_blend_function_rgb_sf = rgb_sf;
  g_gr_state.alpha_blend_function_rgb_df = rgb_df;
  g_gr_state.alpha_blend_function_alpha_sf = alpha_sf;
  g_gr_state.alpha_blend_function_alpha_df = alpha_df;

  // Ugly way to fix the problem with the horses joints in Zelda 64
  if (g_gr_bAlphaFix)
    {
      if (g_gr_state.alpha_combine_other != GR_COMBINE_OTHER_TEXTURE &&
	  g_gr_AlphaTestOn)
	glDisable (GL_ALPHA_TEST);
      else if (g_gr_AlphaTestOn)
	glEnable (GL_ALPHA_TEST);
    }



  if ((rgb_sf == GR_BLEND_ONE) && (rgb_df == GR_BLEND_ZERO))
    {

      glDisable (GL_BLEND);

    }
  else
    {
      if (g_gr_state.alpha_combine_function ==
	  GR_COMBINE_FUNCTION_SCALE_OTHER &&
	  g_gr_state.alpha_combine_other == GR_COMBINE_OTHER_TEXTURE)
	glEnable (GL_TEXTURE_2D);
      glEnable (GL_BLEND);
      switch (rgb_sf)
	{
	case GR_BLEND_ZERO:
	  sfactor = GL_ZERO;
	  break;
	case GR_BLEND_ONE:
	  sfactor = GL_ONE;
	  break;
	case GR_BLEND_PREFOG_COLOR:
	  sfactor = GL_DST_COLOR;	// ???
	  break;
	case GR_BLEND_DST_COLOR:
	  sfactor = GL_DST_COLOR;
	  break;
	case GR_BLEND_ONE_MINUS_DST_COLOR:
	  sfactor = GL_ONE_MINUS_DST_COLOR;
	  break;
	case GR_BLEND_SRC_ALPHA:
	  sfactor = GL_SRC_ALPHA;
	  break;
	case GR_BLEND_ONE_MINUS_SRC_ALPHA:
	  sfactor = GL_ONE_MINUS_SRC_ALPHA;
	  break;
	case GR_BLEND_DST_ALPHA:
	  sfactor = GL_DST_ALPHA;
	  break;
	case GR_BLEND_ONE_MINUS_DST_ALPHA:
	  sfactor = GL_ONE_MINUS_DST_ALPHA;
	  break;
	}

      switch (rgb_df)
	{
	case GR_BLEND_ZERO:
	  dfactor = GL_ZERO;
	  break;
	case GR_BLEND_ONE:
	  dfactor = GL_ONE;
	  break;
	case GR_BLEND_PREFOG_COLOR:
	  dfactor = GL_DST_COLOR;	// ???
	  break;
	case GR_BLEND_DST_COLOR:
	  dfactor = GL_DST_COLOR;
	  break;
	case GR_BLEND_ONE_MINUS_DST_COLOR:
	  dfactor = GL_ONE_MINUS_DST_COLOR;
	  break;
	case GR_BLEND_SRC_ALPHA:
	  dfactor = GL_SRC_ALPHA;
	  break;
	case GR_BLEND_ONE_MINUS_SRC_ALPHA:
	  dfactor = GL_ONE_MINUS_SRC_ALPHA;
	  break;
	case GR_BLEND_DST_ALPHA:
	  dfactor = GL_DST_ALPHA;
	  break;
	case GR_BLEND_ONE_MINUS_DST_ALPHA:
	  dfactor = GL_ONE_MINUS_DST_ALPHA;
	  break;
	}

      glBlendFunc (sfactor, dfactor);



    }
}

FX_ENTRY void FX_CALL
grAlphaCombine (
		 GrCombineFunction_t function, GrCombineFactor_t factor,
		 GrCombineLocal_t local, GrCombineOther_t other,
		 FxBool invert
)
{
  TRACEFUNCTION ("grAlphaCombine");

  g_gr_state.alpha_combine_function = function;
  g_gr_state.alpha_combine_factor = factor;
  g_gr_state.alpha_combine_local = local;
  g_gr_state.alpha_combine_other = other;

  grMegaCombineEverythingSetup (true);
}

FX_ENTRY void FX_CALL
grAlphaControlsITRGBLighting (FxBool enable)
{
  TRACEFUNCTION ("grAlphaControlsITRGBLighting");
  ERROROUT ("grAlphaControlsITRGBLighting");


}

FX_ENTRY void FX_CALL
grAlphaTestFunction (GrCmpFnc_t function)
{
  TRACEFUNCTION ("grAlphaTestFunction");

  g_gr_state.alpha_test_function = function;
  switch (function)
    {
    case GR_CMP_NEVER:
      glAlphaFunc (GL_NEVER, g_gr_state.alpha_test_reference_value / 255.0f);
      glEnable (GL_ALPHA_TEST);
      g_gr_AlphaTestOn = TRUE;
      break;
    case GR_CMP_LESS:
      glAlphaFunc (GL_LESS, (g_gr_state.alpha_test_reference_value - 1) / 255.0f);
      glEnable (GL_ALPHA_TEST);
      g_gr_AlphaTestOn = TRUE;
      break;
    case GR_CMP_EQUAL:
      glAlphaFunc (GL_EQUAL, g_gr_state.alpha_test_reference_value / 255.0f);
      glEnable (GL_ALPHA_TEST);
      ERROROUT ("AlphaTestFunction: EQUAL");
      g_gr_AlphaTestOn = TRUE;
      break;
    case GR_CMP_LEQUAL:
      glAlphaFunc (GL_LEQUAL, (g_gr_state.alpha_test_reference_value - 1) / 255.0f);
      glEnable (GL_ALPHA_TEST);
      g_gr_AlphaTestOn = TRUE;
      break;
    case GR_CMP_GREATER:
      glAlphaFunc (GL_GREATER, (g_gr_state.alpha_test_reference_value + 1) / 255.0f);
      glEnable (GL_ALPHA_TEST);
      g_gr_AlphaTestOn = TRUE;
      break;
    case GR_CMP_NOTEQUAL:
      glAlphaFunc (GL_NOTEQUAL, g_gr_state.alpha_test_reference_value / 255.0f);
      ERROROUT ("AlphaTestFunction: NOTEQUAL");
      glEnable (GL_ALPHA_TEST);
      g_gr_AlphaTestOn = TRUE;
      break;
    case GR_CMP_GEQUAL:
      glAlphaFunc (GL_GEQUAL, (g_gr_state.alpha_test_reference_value + 1) / 255.0f);
      glEnable (GL_ALPHA_TEST);
      g_gr_AlphaTestOn = TRUE;
      break;
    case GR_CMP_ALWAYS:
      glDisable (GL_ALPHA_TEST);
      g_gr_AlphaTestOn = FALSE;
      break;
    }
}

FX_ENTRY void FX_CALL
grAlphaTestReferenceValue (GrAlpha_t value)
{
  //static char buf[256];
  TRACEFUNCTION ("grAlphaTestReferenceValue");

  g_gr_state.alpha_test_reference_value = value;
  grAlphaTestFunction (g_gr_state.alpha_test_function);
}

FX_ENTRY void FX_CALL
grChromakeyMode (GrChromakeyMode_t mode)
{
  TRACEFUNCTION ("grChromakeyMode");

  g_gr_state.chromakey_mode = mode;
}

FX_ENTRY void FX_CALL
grChromakeyValue (GrColor_t value)
{
  TRACEFUNCTION ("grChromakeyValue");

  DWORD cr = grsubConvColor (value);
  g_gr_state.chromakey_value = cr;
}

FX_ENTRY void FX_CALL
grClipWindow (FxU32 minx, FxU32 miny, FxU32 maxx, FxU32 maxy)
{
  TRACEFUNCTION ("grClipWindow");

  g_gr_state.clip_window_minx = minx;
  g_gr_state.clip_window_miny = miny;
  g_gr_state.clip_window_maxx = maxx;
  g_gr_state.clip_window_maxy = maxy;

}

FX_ENTRY void FX_CALL
grColorCombine (
		 GrCombineFunction_t function, GrCombineFactor_t factor,
		 GrCombineLocal_t local, GrCombineOther_t other,
		 FxBool invert)
{
  TRACEFUNCTION ("grColorCombine");


  g_gr_state.color_combine_function = function;
  g_gr_state.color_combine_factor = factor;
  g_gr_state.color_combine_local = local;
  g_gr_state.color_combine_other = other;
  g_gr_state.color_combine_invert = invert;

  /*if (g_gr_state.color_combine_function == GR_COMBINE_FUNCTION_SCALE_OTHER_ADD_LOCAL_ALPHA)
     ;
     else
     ;

     if (g_gr_state.color_combine_factor == GR_COMBINE_FACTOR_ONE) {
     } else {
     } */


  grMegaCombineEverythingSetup (false);

  g_gr_tmu[0].texture_source_start_address_old = -1;
}

FX_ENTRY void FX_CALL
grColorMask (FxBool rgb, FxBool a)
{
  TRACEFUNCTION ("grColorMask");

  g_gr_state.color_mask_rgb = rgb;
  g_gr_state.color_mask_a = a;
  glColorMask(rgb, rgb, rgb, a);
  /*DWORD dwMask = 0;
     if (rgb == FXTRUE) dwMask |= 0x00ffffff;
     if (a == FXTRUE) dwMask |= 0xff000000; */

}

FX_ENTRY void FX_CALL
grCullMode (GrCullMode_t mode)
{
  TRACEFUNCTION ("grCullMode");

  g_gr_state.cull_mode = mode;
  switch (mode)
    {
    case GR_CULL_DISABLE:
      glDisable (GL_CULL_FACE);
      break;
    case GR_CULL_NEGATIVE:
      glEnable (GL_CULL_FACE);
      glCullFace (GL_BACK);
      break;
    case GR_CULL_POSITIVE:
      glEnable (GL_CULL_FACE);
      glCullFace (GL_FRONT);
      break;
    }
}

FX_ENTRY void FX_CALL
grConstantColorValue (GrColor_t value)
{
  TRACEFUNCTION ("grConstantColorValue");

  DWORD cr = grsubConvColor (value);
  g_gr_state.constant_color_value = cr;
  g_gr_state.constant_color_value_color = cr & 0x00ffffff;
  g_gr_state.constant_color_value_alpha = cr & 0xff000000;
}

FX_ENTRY void FX_CALL
grConstantColorValue4 (float a, float r, float g, float b)
{
  TRACEFUNCTION ("grConstantColorValue4");

  DWORD value = (((DWORD) a & 0xff) << 24) |
  (((DWORD) r & 0xff) << 16) |
  (((DWORD) g & 0xff) << 8) |
  (((DWORD) b & 0xff) << 0);
  DWORD cr = grsubConvColor (value);

  g_gr_state.constant_color_value = cr;
  g_gr_state.constant_color_value_color = cr & 0x00ffffff;
  g_gr_state.constant_color_value_alpha = cr & 0xff000000;
}

FX_ENTRY void FX_CALL
grDepthBiasLevel (FxI16 level)
{
  TRACEFUNCTION ("grDepthBiasLevel");
  ERROROUT ("grDepthBiasLevel");
  g_gr_state.depth_bias_level = level;
}

FX_ENTRY void FX_CALL
grDepthBufferFunction (GrCmpFnc_t function)
{
  TRACEFUNCTION ("grDepthBufferFunction");

  g_gr_state.depth_buffer_function = function;
  switch (function)
    {
    case GR_CMP_NEVER:
      glDepthFunc (GL_NEVER);
//              ERROROUT("DepthBufferFunction: NEVER");
      break;
    case GR_CMP_LESS:
      glDepthFunc (GL_LESS);
//              ERROROUT("DepthBufferFunction: LESS");
      break;
    case GR_CMP_EQUAL:
      glDepthFunc (GL_EQUAL);
//              ERROROUT("DepthBufferFunction: EQUAL");
      break;
    case GR_CMP_LEQUAL:
      glDepthFunc (GL_LEQUAL);
//              ERROROUT("DepthBufferFunction: LEQUAL");
      break;
    case GR_CMP_GREATER:
      glDepthFunc (GL_GREATER);
//              ERROROUT("DepthBufferFunction: GREATER");
      break;
    case GR_CMP_NOTEQUAL:
      glDepthFunc (GL_NOTEQUAL);
//              ERROROUT("DepthBufferFunction: NOTEQUAL");
      break;
    case GR_CMP_GEQUAL:
      glDepthFunc (GL_GEQUAL);
//              ERROROUT("DepthBufferFunction: GEQUAL");
      break;
    case GR_CMP_ALWAYS:
      glDepthFunc (GL_ALWAYS);
//              ERROROUT("DepthBufferFunction: ALWAYS");
      break;
    }
}

FX_ENTRY void FX_CALL
grDepthBufferMode (GrDepthBufferMode_t mode)
{
  TRACEFUNCTION ("grDepthBufferMode");
//      ERROROUT("grDepthBufferMode");

  g_gr_state.depth_buffer_mode = mode;

  switch (mode)
    {
    case GR_DEPTHBUFFER_DISABLE:
      glDisable (GL_DEPTH_TEST);
      break;
    case GR_DEPTHBUFFER_ZBUFFER:
      glEnable (GL_DEPTH_TEST);
      break;
    case GR_DEPTHBUFFER_WBUFFER:
      glEnable (GL_DEPTH_TEST);
      break;
    default:
      ERROROUT ("COMPARE TO BIAS");
      break;
    }

}

FX_ENTRY void FX_CALL
grDepthMask (FxBool mask)
{
  TRACEFUNCTION ("grDepthMask");

  g_gr_state.depth_mask = mask;
  glDepthMask (mask);
}

FX_ENTRY void FX_CALL
grDisableAllEffects (void)
{
  TRACEFUNCTION ("grDisableAllEffects");

  grAlphaBlendFunction (GR_BLEND_ONE, GR_BLEND_ZERO, GR_BLEND_ONE, GR_BLEND_ZERO);
  grAlphaTestFunction (GR_CMP_ALWAYS);
  grChromakeyMode (GR_CHROMAKEY_DISABLE);
  grDepthBufferMode (GR_DEPTHBUFFER_DISABLE);
  grFogMode (GR_FOG_DISABLE);
}

FX_ENTRY void FX_CALL
grDitherMode (GrDitherMode_t mode)
{
  TRACEFUNCTION ("grDitherMode");

  g_gr_state.dither_mode = mode;
}

FX_ENTRY void FX_CALL
grFogColorValue (GrColor_t fogcolor)
{
  TRACEFUNCTION ("grFogColorValue");

  g_gr_state.fog_color = fogcolor;
  DWORD cr = grsubConvColor (fogcolor);
  g_gr_state.fog_d3dcolor = cr;
  grFogMode (g_gr_state.fog_mode);

}

FX_ENTRY void FX_CALL
grFogMode (GrFogMode_t mode)
{
  static GLfloat color[4];
  TRACEFUNCTION ("grFogMode");
  float *n, *f;

  if (g_gr_bFogMode == GLID3D_FOG_OFF)
    return;

  g_gr_state.fog_mode = mode;
  color[0] = (float) ((g_gr_state.fog_d3dcolor >> 16) & 0xff) / 255.0f;
  color[1] = (float) ((g_gr_state.fog_d3dcolor >> 8) & 0xff) / 255.0f;
  color[2] = (float) ((g_gr_state.fog_d3dcolor) & 0xff) / 255.0f;
  color[3] = (float) ((g_gr_state.fog_d3dcolor >> 24) & 0xff) / 255.0f;
  if (mode == GR_FOG_DISABLE)
    {
      glDisable (GL_FOG);
      return;
    }

  if (g_gr_state.fog_mode & GR_FOG_WITH_TABLE)
    {
      INT nFogTableValidate = (INT) g_gr_fogtable[GLID3D_FOGTABLE_VALIDATE];
      if (nFogTableValidate == GLID3D_FOGTABLE_VALIDATE_VALUE)
	{
	  INT nFogMode = (INT) g_gr_fogtable[GLID3D_FOGTABLE_MODE];
	  switch (nFogMode)
	    {
	    case GLID3D_FOGMODE_EXP:
	      {
		glEnable (GL_FOG);
		glFogi (GL_FOG_MODE, GL_EXP);
		glFogf (GL_FOG_DENSITY, g_gr_fogtable[GLID3D_FOGTABLE_DENSITY]);
		glFogfv (GL_FOG_COLOR, color);
		break;
	      }
	    case GLID3D_FOGMODE_EXP2:
	      {
		glEnable (GL_FOG);
		glFogi (GL_FOG_MODE, GL_EXP2);
		glFogf (GL_FOG_DENSITY, g_gr_fogtable[GLID3D_FOGTABLE_DENSITY]);
		glFogfv (GL_FOG_COLOR, color);
		break;
	      }
	    case GLID3D_FOGMODE_LINEAR:
	      {
		n = (float *) &g_gr_fogtable[GLID3D_FOGTABLE_NEARZ];
		f = (float *) &g_gr_fogtable[GLID3D_FOGTABLE_FARZ];
		if (g_gr_bUseZInverse)
		  {
		    if (g_gr_bFogMode == GLID3D_FOG_MODE1)
		      {
			if (g_gr_bPrecisionFix)
			  {
			    (*n) = 1.0 - 0.1 / (*n);
			    (*f) = 1.0 - 0.1 / (*f);
			  }
			else
			  {
			    (*n) = 1.0 - 1 / (*n);
			    (*f) = 1.0 - 1 / (*f);
			  }
		      }
		    else
		      {
			if (g_gr_bPrecisionFix)
			  {
			    (*n) = (*n);
			    (*f) = (*f);
			  }
			else
			  {
			    (*n) = (*n);
			    (*f) = (*f);
			  }
		      }
		  }
		else
		  {
		    (*n) = (*n);
		    (*f) = (*f);
		  }


		glEnable (GL_FOG);
		glFogi (GL_FOG_MODE, GL_LINEAR);
		glFogf (GL_FOG_START, *n);
		glFogf (GL_FOG_END, *f);
		glFogfv (GL_FOG_COLOR, color);
		break;
	      }
	    }
	}
      else
	{
	  // Invalid fogtable it was generated by non glide function
	  glDisable (GL_FOG);
	}
    }
//      else 
//              glDisable(GL_FOG);
}

FX_ENTRY void FX_CALL
grFogTable (const GrFog_t ft[GR_FOG_TABLE_SIZE])
{
  TRACEFUNCTION ("grFogTable");

  if (g_gr_bFogMode == GLID3D_FOG_OFF)
    return;

  CopyMemory (g_gr_fogtable, ft, sizeof (BYTE) * GR_FOG_TABLE_SIZE);

  grFogMode (g_gr_state.fog_mode);
}

FX_ENTRY void FX_CALL
grGammaCorrectionValue (float value)
{
  TRACEFUNCTION ("grGammaCorrectionValue");

  if (g_gr_bNoGammaControl)
    return;

  value += g_gr_fGammaBase;
  g_gr_fGamma = value;

  /*INT i;
     for (i = 0; i < 256; i++) {
     INT nRamp;
     if (value >= 0.0f) {
     nRamp = (INT)((FLOAT)i * (FLOAT)value * (FLOAT)0xffff / 255.0f);
     nRamp += g_gr_nContrast;
     if (nRamp < 0) nRamp = 0;
     else if (nRamp > 0xffff) nRamp = 0xffff;
     } else {
     nRamp = 0;
     }
     } */

}

FX_ENTRY void FX_CALL
grSplash (float x, float y, float width, float height, FxU32 frame)
{
  TRACEFUNCTION ("grSplash");

  if (frame == 0)
    {
      ERROROUT ("INSPLASH");
    }
}

/////////////////////////////////////////////////////////////////////////////
